use std::borrow::Cow;

use validator::Validate;

use super::response::{ResData, ResErr, ResResult};

pub fn validate_params(params: &impl Validate, _local: &str) -> ResResult<ResData<()>> {
    params.validate().map_err(|e| {
        let mut errors = vec![];
        for (_, err) in e.errors().iter() {
            match err {
                validator::ValidationErrorsKind::Struct(err) => {
                    err.field_errors().iter().for_each(|(_field, errs)| {
                        errs.iter().for_each(|e| {
                            if let Some(Cow::Owned(msg)) = e.message.clone() {
                                errors.push(msg);
                            }
                        });
                    });
                },
                validator::ValidationErrorsKind::List(err) => {
                    for (_, err) in err.iter() {
                        err.field_errors().iter().for_each(|(_field, errs)| {
                            errs.iter().for_each(|e| {
                                if let Some(Cow::Owned(msg)) = e.message.clone() {
                                    errors.push(msg);
                                }
                            });
                        });
                    }
                },
                validator::ValidationErrorsKind::Field(err) => {
                    err.iter().for_each(|e| {
                        if let Some(Cow::Owned(msg)) = e.message.clone() {
                            errors.push(msg);
                        }
                    });
                },
            };
        }
        ResErr::params(errors.join(";"))
    })?;
    Result::Ok(ResData::none())
}